@@ -29,7 +29,49 @@ require "gettext/po"
 module GetText
   module Tools
     class MsgMerge
-      class Merger #:nodoc:
+      class << self
+        # (see #run)
+        #
+        # This method is provided just for convenience. It equals to
+        # `new.run(*command_line)`.
+        def run(*command_line)
+          new.run(*command_line)
+        end
+      end
+
+      # Merge a po-file inluding translated messages and a new pot-file.
+      #
+      # @param [Array<String>] command_line
+      #   command line arguments for rmsgmerge.
+      # @return [void]
+      def run(*command_line)
+        config = Config.new
+        config.parse(command_line)
+
+        parser = POParser.new
+        parser.ignore_fuzzy = false
+        definition_po = PO.new
+        reference_pot = PO.new
+        parser.parse_file(config.definition_po, definition_po)
+        parser.parse_file(config.reference_pot, reference_pot)
+
+        merger = Merger.new(reference_pot, definition_po, config)
+        result = merger.merge
+        result.order = config.order
+        p result if $DEBUG
+        print result.generate_po if $DEBUG
+
+        if config.output.is_a?(String)
+          File.open(File.expand_path(config.output), "w+") do |file|
+            file.write(result.to_s(config.po_format_options))
+          end
+        else
+          puts(result.to_s(config.po_format_options))
+        end
+      end
+
+      # @private
+      class Merger
         # Merge the reference with the definition: take the #. and
         #  #: comments from the reference, take the # comments from
         # the definition, take the msgstr from the definition.  Add
@@ -38,56 +80,61 @@ module GetText
         POT_DATE_EXTRACT_RE = /POT-Creation-Date:\s*(.*)?\s*$/
         POT_DATE_RE = /POT-Creation-Date:.*?$/
 
-        def merge(definition, reference)
-          result = GetText::PO.new
-
-          translated_entries = definition.reject do |entry|
+        def initialize(reference, definition, config)
+          @reference = reference
+          @definition = definition
+          @translated_entries = @definition.reject do |entry|
             entry.msgstr.nil?
           end
+          @config = config
+        end
 
-          reference.each do |entry|
-            msgid = entry.msgid
-            msgctxt = entry.msgctxt
-            id = [msgctxt, msgid]
+        def merge
+          result = GetText::PO.new
 
-            if definition.has_key?(*id)
-              result[*id] = merge_entry(definition[*id], entry)
-              next
-            end
-
-            if msgctxt.nil?
-              same_msgid_entry = find_by_msgid(translated_entries, msgid)
-              if not same_msgid_entry.nil? and not same_msgid_entry.msgctxt.nil?
-                result[nil, msgid] = merge_fuzzy_entry(same_msgid_entry, entry)
-                next
-              end
-            end
-
-            fuzzy_entry = find_fuzzy_entry(translated_entries, msgid, msgctxt)
-            unless fuzzy_entry.nil?
-              result[*id] = merge_fuzzy_entry(fuzzy_entry, entry)
-              next
-            end
-
-            result[*id] = entry
+          @reference.each do |entry|
+            id = [entry.msgctxt, entry.msgid]
+            result[*id] = merge_definition(entry)
           end
 
-          add_obsolete_entry(result, translated_entries)
+          add_obsolete_entry(result)
           result
         end
 
-        def merge_entry(definition_entry, reference_entry)
-          if definition_entry.msgid.empty? and definition_entry.msgctxt.nil?
-            new_header = merge_header(definition_entry, reference_entry)
-            return new_header
+        private
+        def merge_definition(entry)
+          msgid = entry.msgid
+          msgctxt = entry.msgctxt
+          id = [msgctxt, msgid]
+
+          if @definition.has_key?(*id)
+            return merge_entry(entry, @definition[*id])
           end
 
-          if definition_entry.flag == "fuzzy"
-            entry = definition_entry
-            entry.flag = "fuzzy"
-            return entry
+          return entry unless @config.enable_fuzzy_matching?
+
+          if msgctxt.nil?
+            same_msgid_entry = find_by_msgid(@translated_entries, msgid)
+            if same_msgid_entry and same_msgid_entry.msgctxt
+              return merge_fuzzy_entry(entry, same_msgid_entry)
+            end
           end
 
+          fuzzy_entry = find_fuzzy_entry(@translated_entries, msgid, msgctxt)
+          if fuzzy_entry
+            return merge_fuzzy_entry(entry, fuzzy_entry)
+          end
+
+          entry
+        end
+
+        def merge_entry(reference_entry, definition_entry)
+          if definition_entry.header?
+            return merge_header(reference_entry, definition_entry)
+          end
+
+          return definition_entry if definition_entry.flag == "fuzzy"
+
           entry = reference_entry
           entry.translator_comment = definition_entry.translator_comment
           entry.previous = nil
@@ -100,7 +147,7 @@ module GetText
           entry
         end
 
-        def merge_header(old_header, new_header)
+        def merge_header(new_header, old_header)
           header = old_header
           if POT_DATE_EXTRACT_RE =~ new_header.msgstr
             create_date = $1
@@ -121,8 +168,8 @@ module GetText
           same_msgid_entries.first
         end
 
-        def merge_fuzzy_entry(fuzzy_entry, entry)
-          merged_entry = merge_entry(fuzzy_entry, entry)
+        def merge_fuzzy_entry(entry, fuzzy_entry)
+          merged_entry = merge_entry(entry, fuzzy_entry)
           merged_entry.flag = "fuzzy"
           merged_entry
         end
@@ -155,40 +202,38 @@ module GetText
           Text::Levenshtein.distance(source, destination) / max_size.to_f
         end
 
-        def add_obsolete_entry(result, definition)
-          obsolete_entry = generate_obsolete_entry(result, definition)
-          unless obsolete_entry.nil?
-            result[:last] = obsolete_entry
-          end
-          result
+        def add_obsolete_entry(result)
+          obsolete_entry = generate_obsolete_entry(result)
+          return if obsolete_entry.nil?
+
+          result[:last] = obsolete_entry
         end
 
-        def generate_obsolete_entry(result, definition)
-          obsolete_entry = nil
-
-          obsolete_entries = extract_obsolete_entries(result, definition)
-          unless obsolete_entries.empty?
-            obsolete_comment = []
-
-            obsolete_entries.each do |entry|
-              obsolete_comment << entry.to_s
-            end
-            obsolete_entry = POEntry.new(:normal)
-            obsolete_entry.msgid = :last
-            obsolete_entry.comment = obsolete_comment.join("\n")
+        def generate_obsolete_entry(result)
+          obsolete_entries = extract_obsolete_entries(result)
+          obsolete_comments = obsolete_entries.collect do |entry|
+            entry.to_s
           end
+
+          return nil if obsolete_comments.empty?
+
+          obsolete_entry = POEntry.new(:normal)
+          obsolete_entry.msgid = :last
+          obsolete_entry.comment = obsolete_comments.join("\n")
           obsolete_entry
         end
 
-        def extract_obsolete_entries(result, definition)
-          obsolete_entries = []
-          definition.each do |entry|
-            id = [entry.msgctxt, entry.msgid]
-            if not result.has_key?(*id) and not entry.msgid == :last
-              obsolete_entries << entry
+        def extract_obsolete_entries(result)
+          @definition.find_all do |entry|
+            if entry.obsolete?
+              true
+            elsif entry.msgstr.nil?
+              false
+            else
+              id = [entry.msgctxt, entry.msgid]
+              not result.has_key?(*id)
             end
           end
-          obsolete_entries
         end
       end
 
@@ -199,13 +244,16 @@ module GetText
         bindtextdomain("gettext")
 
         attr_accessor :definition_po, :reference_pot
-        attr_accessor :output, :fuzzy, :update
+        attr_accessor :output, :update
         attr_accessor :order
         attr_accessor :po_format_options
 
         # update mode options
         attr_accessor :backup, :suffix
 
+        # (#see #enable_fuzzy_matching?)
+        attr_writer :enable_fuzzy_matching
+
         # The result is written back to def.po.
         #       --backup=CONTROL        make a backup of def.po
         #       --suffix=SUFFIX         override the usual backup suffix
@@ -228,7 +276,7 @@ module GetText
           @po_format_options = {
             :max_line_width => POEntry::Formatter::DEFAULT_MAX_LINE_WIDTH,
           }
-          @fuzzy = nil
+          @enable_fuzzy_matching = true
           @update = nil
           @backup = ENV["VERSION_CONTROL"]
           @suffix = ENV["SIMPLE_BACKUP_SUFFIX"] || "~"
@@ -248,6 +296,11 @@ module GetText
           @output = @definition_po if @update
         end
 
+        # @return [Bool] true if fuzzy matching is enabled, false otherwise.
+        def enable_fuzzy_matching?
+          @enable_fuzzy_matching
+        end
+
         private
         def create_option_parser
           parser = OptionParser.new
@@ -312,7 +365,11 @@ module GetText
             @po_format_options[:max_line_width] = max_line_width
           end
 
-          #parser.on("-F", "--fuzzy-matching")
+          parser.on("--[no-]fuzzy-matching",
+                    _("Disable fuzzy matching"),
+                    _("(enable)")) do |boolean|
+            @enable_fuzzy_matching = boolean
+          end
 
           parser.on("-h", "--help", _("Display this help and exit")) do
             puts(parser.help)
@@ -328,47 +385,6 @@ module GetText
           parser
         end
       end
-
-      class << self
-        # (see #run)
-        #
-        # This method is provided just for convenience. It equals to
-        # `new.run(*command_line)`.
-        def run(*command_line)
-          new.run(*command_line)
-        end
-      end
-
-      # Merge a po-file inluding translated messages and a new pot-file.
-      #
-      # @param [Array<String>] command_line
-      #   command line arguments for rmsgmerge.
-      # @return [void]
-      def run(*command_line)
-        config = Config.new
-        config.parse(command_line)
-
-        parser = POParser.new
-        parser.ignore_fuzzy = false
-        definition_po = PO.new
-        reference_pot = PO.new
-        parser.parse_file(config.definition_po, definition_po)
-        parser.parse_file(config.reference_pot, reference_pot)
-
-        merger = Merger.new
-        result = merger.merge(definition_po, reference_pot)
-        result.order = config.order
-        p result if $DEBUG
-        print result.generate_po if $DEBUG
-
-        if config.output.is_a?(String)
-          File.open(File.expand_path(config.output), "w+") do |file|
-            file.write(result.to_s(config.po_format_options))
-          end
-        else
-          puts(result.to_s(config.po_format_options))
-        end
-      end
     end
   end
 end
