@@ -24,6 +24,7 @@ require "pathname"
 require "optparse"
 require "locale"
 require "gettext"
+require "gettext/po"
 
 module GetText
   module Tools
@@ -78,11 +79,12 @@ module GetText
         @input_files = nil
         @output = nil
 
-        @package_name = nil
-        @package_version = nil
-        @msgid_bugs_address = nil
-        @copyright_holder = nil
-        @output_encoding = nil
+        @package_name = "PACKAGE"
+        @package_version = "VERSION"
+        @msgid_bugs_address = ""
+        @copyright_holder = "THE PACKAGE'S COPYRIGHT HOLDER"
+        @copyright_year = "YEAR"
+        @output_encoding = "UTF-8"
 
         @parse_options = {}
 
@@ -146,7 +148,6 @@ module GetText
       def run(*options)  # :nodoc:
         check_command_line_options(*options)
 
-        @output_encoding ||= "UTF-8"
         pot = generate_pot(@input_files)
 
         if @output.is_a?(String)
@@ -181,9 +182,9 @@ module GetText
       def header_comment
         <<-COMMENT
 SOME DESCRIPTIVE TITLE.
-Copyright (C) YEAR #{@copyright_holder}
+Copyright (C) #{@copyright_year} #{@copyright_holder}
 This file is distributed under the same license as the #{@package_name} package.
-FIRST AUTHOR <EMAIL@ADDRESS>, YEAR.
+FIRST AUTHOR <EMAIL@ADDRESS>, #{@copyright_year}.
 
        COMMENT
       end
@@ -232,11 +233,6 @@ Plural-Forms: nplurals=INTEGER; plural=EXPRESSION;
 
         @input_files = input_files
         @output = output
-
-        @package_name ||= "PACKAGE"
-        @package_version ||= "VERSION"
-        @msgid_bugs_address ||= ""
-        @copyright_holder ||= "THE PACKAGE'S COPYRIGHT HOLDER"
       end
 
       def parse_arguments(*options) #:nodoc:
@@ -256,28 +252,39 @@ Plural-Forms: nplurals=INTEGER; plural=EXPRESSION;
           output = out
         end
 
-        parser.on("--package-name=PACKAGE",
-                  _("set package name in output")) do |out|
-          @package_name = out
+        parser.on("--package-name=NAME",
+                  _("set package name in output"),
+                  "(#{@package_name})") do |name|
+          @package_name = name
         end
 
         parser.on("--package-version=VERSION",
-                  _("set package version in output")) do |out|
-          @package_version = out
+                  _("set package version in output"),
+                  "(#{@package_version})") do |version|
+          @package_version = version
         end
 
         parser.on("--msgid-bugs-address=EMAIL",
-                  _("set report address for msgid bugs")) do |out|
-          @msgid_bugs_address = out
+                  _("set report e-mail address for msgid bugs"),
+                  "(#{@msgid_bugs_address})") do |address|
+          @msgid_bugs_address = address
         end
 
-        parser.on("--copyright-holder=STRING",
-                  _("set copyright holder in output")) do |out|
-          @copyright_holder = out
+        parser.on("--copyright-holder=HOLDER",
+                  _("set copyright holder in output"),
+                  "(#{@copyright_holder})") do |holder|
+          @copyright_holder = holder
+        end
+
+        parser.on("--copyright-year=YEAR",
+                  _("set copyright year in output"),
+                  "(#{@copyright_year})") do |year|
+          @copyright_year = year
         end
 
         parser.on("--output-encoding=ENCODING",
-                  _("set encoding for output")) do |encoding|
+                  _("set encoding for output"),
+                  "(#{@output_encoding})") do |encoding|
           @output_encoding = encoding
         end
 
