@@ -23,36 +23,83 @@ require 'gettext/tools/msgmerge'
 class TestToolsMsgMerge < Test::Unit::TestCase
   class TestMerger < self
     def setup
-      @merger = GetText::Tools::MsgMerge::Merger.new
       @po = GetText::PO.new
       @pot = GetText::PO.new
+      @config = GetText::Tools::MsgMerge::Config.new
     end
 
-    def test_add_entry
-      @po["hello"] = "bonjour"
-      @pot["hello"] = "bonjour"
-      @pot["he"] = "il"
-      merged_po = @merger.merge(@po, @pot)
-
-      assert_equal("bonjour", merged_po["hello"].msgstr)
-      assert_equal("il", merged_po["he"].msgstr)
+    def parse_po(po, output)
+      parser = GetText::POParser.new
+      parser.report_warning = false
+      parser.parse(po, output)
     end
 
-    def test_existing_obsolete_entry
-      @po["hello"] = "bonjour"
-      @po[:last] = generate_entry(:msgid => :last,
-                                  :comment => "#~ obsolete_entry")
-      @pot["hello"] = "bonjour"
-      merged_po = @merger.merge(@po, @pot)
+    def merge
+      merger = GetText::Tools::MsgMerge::Merger.new(@pot, @po, @config)
+      merger.merge
+    end
 
-      assert_equal("bonjour", merged_po["hello"].msgstr)
-      assert_nil(merged_po[:last])
+    class TestUpdate < self
+      def test_msgstr
+        pot = <<-POT
+msgid "hello"
+msgstr ""
+        POT
+        po = <<-PO
+msgid "hello"
+msgstr "bonjour"
+        PO
+        merged_po = merge(pot, po)
+
+        assert_equal(<<-PO, merged_po)
+msgid "hello"
+msgstr "bonjour"
+        PO
+      end
+
+      private
+      def merge(pot, po)
+        parse_po(pot, @pot)
+        parse_po(po, @po)
+        super().to_s
+      end
+    end
+
+    class TestObosleteEntry < self
+      def test_in_po
+        pot = <<-POT
+msgid "hello"
+msgstr ""
+        POT
+        po = <<-PO
+msgid "hello"
+msgstr "bonjour"
+
+#~ msgid "he"
+#~ msgstr "il"
+        PO
+
+        assert_equal(<<-PO, merge(pot, po))
+msgid "hello"
+msgstr "bonjour"
+
+#~ msgid "he"
+#~ msgstr "il"
+        PO
+      end
+
+      private
+      def merge(pot, po)
+        parse_po(pot, @pot)
+        parse_po(po, @po)
+        super().to_s
+      end
     end
 
     def test_different_msgstr
       @po["hello"] = "salut"
       @pot["hello"] = "bonjour"
-      merged_po = @merger.merge(@po, @pot)
+      merged_po = merge
 
       assert_equal("salut", merged_po["hello"].msgstr)
     end
@@ -66,7 +113,7 @@ class TestToolsMsgMerge < Test::Unit::TestCase
                                      :msgstr => "",
                                      :translator_comment => "It's comments")
 
-      merged_po = @merger.merge(@po, @pot)
+      merged_po = merge
       assert_equal("bonjour", merged_po["hello"].msgstr)
       assert_equal("comment", merged_po["hello"].translator_comment)
     end
@@ -80,7 +127,7 @@ class TestToolsMsgMerge < Test::Unit::TestCase
                                      :msgstr => "",
                                      :extracted_comment => "extracted comments")
 
-      merged_po = @merger.merge(@po, @pot)
+      merged_po = merge
       assert_equal("bonjour", merged_po["hello"].msgstr)
       assert_equal("extracted comments", merged_po["hello"].extracted_comment)
     end
@@ -96,7 +143,7 @@ class TestToolsMsgMerge < Test::Unit::TestCase
                                      :msgstr => "",
                                      :references => pot_references)
 
-      merged_po = @merger.merge(@po, @pot)
+      merged_po = merge
       assert_equal("bonjour", merged_po["hello"].msgstr)
       assert_equal(pot_references, merged_po["hello"].references)
     end
@@ -110,7 +157,7 @@ class TestToolsMsgMerge < Test::Unit::TestCase
                                      :msgstr => "",
                                      :flag => "no-c-format")
 
-      merged_po = @merger.merge(@po, @pot)
+      merged_po = merge
       assert_equal("bonjour", merged_po["hello"].msgstr)
       assert_equal("no-c-format", merged_po["hello"].flag)
     end
@@ -123,7 +170,7 @@ class TestToolsMsgMerge < Test::Unit::TestCase
       @pot["hello"] = generate_entry(:msgid => "hello",
                                      :msgstr => "")
 
-      merged_po = @merger.merge(@po, @pot)
+      merged_po = merge
       assert_equal("bonjour", merged_po["hello"].msgstr)
       assert_equal(nil, merged_po["hello"].previous)
     end
@@ -134,7 +181,7 @@ class TestToolsMsgMerge < Test::Unit::TestCase
                                       :msgstr => nil)
         @pot["hello"] = generate_entry(:msgid => "hello",
                                        :msgstr => nil)
-        merged_po = @merger.merge(@po, @pot)
+        merged_po = merge
         assert_true(merged_po.has_key?("hello"))
         assert_nil(merged_po["hello"].flag)
       end
@@ -149,7 +196,7 @@ class TestToolsMsgMerge < Test::Unit::TestCase
                                   :translator_comment => "uninitialized comment",
                                   :flag => "fuzzy")
 
-        merged_po = @merger.merge(@po, @pot)
+        merged_po = merge
         assert_equal("header\nentry", merged_po[""].msgstr)
         assert_equal("header comment", merged_po[""].translator_comment)
         assert_equal(nil, merged_po[""].flag)
@@ -168,7 +215,7 @@ class TestToolsMsgMerge < Test::Unit::TestCase
                                   :translator_comment => "header comment",
                                   :flag => "fuzzy")
 
-        merged_po = @merger.merge(@po, @pot)
+        merged_po = merge
         assert_equal(pot_creation_date, merged_po[""].msgstr)
       end
     end
@@ -180,7 +227,7 @@ class TestToolsMsgMerge < Test::Unit::TestCase
                                                 :msgstr => "salut")
         @pot["hello"] = generate_entry(:msgid => "hello",
                                        :msgstr => "")
-        merged_po = @merger.merge(@po, @pot)
+        merged_po = merge
 
         assert_false(merged_po.has_key?("normal", "hello"))
         assert_true(merged_po.has_key?("hello"))
@@ -195,7 +242,7 @@ class TestToolsMsgMerge < Test::Unit::TestCase
         @pot["he"] = generate_entry(:msgid => "he",
                                     :msgid_plural => "they",
                                     :msgstr => "")
-        merged_po = @merger.merge(@po, @pot)
+        merged_po = merge
 
         assert_equal("il\000ils", merged_po["he"].msgstr)
         assert_equal("they", merged_po["he"].msgid_plural)
@@ -205,7 +252,7 @@ class TestToolsMsgMerge < Test::Unit::TestCase
       def test_fuzzy_matching_entry
         @po["helol"] = "bonjour"
         @pot["hello"] = ""
-        merged_po = @merger.merge(@po, @pot)
+        merged_po = merge
 
         assert_false(merged_po.has_key?("helol"))
         assert_true(merged_po.has_key?("hello"))
@@ -221,7 +268,7 @@ class TestToolsMsgMerge < Test::Unit::TestCase
         @pot["hello"] = generate_entry(:msgid => "hello",
                                        :msgstr => "")
 
-        merged_po = @merger.merge(@po, @pot)
+        merged_po = merge
         assert_equal("bonjuor", merged_po["hello"].msgstr)
         assert_equal("fuzzy", merged_po["hello"].flag)
       end
@@ -230,7 +277,7 @@ class TestToolsMsgMerge < Test::Unit::TestCase
     def test_obsolete_entry
       @po["hello"] = "bonjour"
       @pot["hi"] = "salut"
-      merged_po = @merger.merge(@po, @pot)
+      merged_po = merge
 
       assert_equal("salut", merged_po["hi"].msgstr)
       assert_false(merged_po.has_key?("hello"))
@@ -587,5 +634,61 @@ msgstr "Translated Hello World. This translation is very long. Yes! Very long tr
         PO
       end
     end
+
+    class TestFuzzyMatching < self
+      def pot_content
+        <<-POT
+msgid "Hello"
+msgstr ""
+        POT
+      end
+
+      def po_content
+        <<-PO
+msgid "Hello!"
+msgstr "Bonjour!"
+        PO
+      end
+
+      def test_default
+        @msgmerge.run("--update",
+                      @po_file_path, @pot_file_path)
+        assert_equal(<<-PO, File.read(@po_file_path))
+#, fuzzy
+msgid "Hello"
+msgstr "Bonjour!"
+
+#~ msgid "Hello!"
+#~ msgstr "Bonjour!"
+        PO
+      end
+
+      def test_fuzzy_matching
+        @msgmerge.run("--update",
+                      "--fuzzy-matching",
+                      @po_file_path, @pot_file_path)
+        assert_equal(<<-PO, File.read(@po_file_path))
+#, fuzzy
+msgid "Hello"
+msgstr "Bonjour!"
+
+#~ msgid "Hello!"
+#~ msgstr "Bonjour!"
+        PO
+      end
+
+      def test_no_fuzzy_matching
+        @msgmerge.run("--update",
+                      "--no-fuzzy-matching",
+                      @po_file_path, @pot_file_path)
+        assert_equal(<<-PO, File.read(@po_file_path))
+msgid "Hello"
+msgstr ""
+
+#~ msgid "Hello!"
+#~ msgstr "Bonjour!"
+        PO
+      end
+    end
   end
 end
